% *************************************************************************
%
%                               LMS versus KLMS
%                         Test 1 for MLSP'2013 tutorial
%                               
%
% contact: jie.chen@unice.fr ; cedric.richard@unice.fr
% version: 7 september 2013
% 
% *************************************************************************

clc
close all;
clear all;


%%

%----------------------------------------------
%
% Signal characteristics
%
%----------------------------------------------

% Input signal std, additive noise std
su = sqrt(0.0156);
rho = 0.5;
q = 2;
R = su^2 * [1 rho ; rho 1];
sw = 1e-3;

% Dictionary length, kernel bandwidth
M = 7;
xi = 0.1;


%%

%----------------------------------------------
%
% Part 2: Monte-Carlo simulations
%
%----------------------------------------------


% Filter step size
eta = 0.08;
mu = 0.1;
thresh = 0.4;

% Number of realizations
rlz = 30;

% Signal length
N = 3000;

% Initializations of variables: KLMS
Ee2 = zeros(1,N);
Ed2 = 0;
Ekd = 0;
Eyn = zeros(N+1,1);

% Initializations of variables: LMS
e2 = zeros(N,1);
e2av = e2;
exe2 = e2;
exe2av = e2;


numdic = 0;
% Loop over Monte Carlo simulations
sb=waitbar(0,'Learning curve calculation under progress');
for ind = 1:rlz
    
    % Filter initializations
    e = zeros(1,N);   
    alpha = 0;
    w = zeros(2,1);
    
    % Initializations of the input and output signals
    un = zeros(2,N);
    yn = zeros(1,N);
    dn = zeros(1,N);
    
    % KLMS dictionary initialization
    ut = su*randn;
    Dic = [ut; rho*ut + sqrt(1-rho^2)*su*randn];
 
    % Loop for signal filtering
    for n = 3:N
        
        % Signal generation
        un(1,n) = su*randn;
        un(2,n) = rho*un(1,n) + sqrt(1-rho^2)*su*randn;
         yn(n) = un(1,n) + 0.5*un(2,n) - 0.2*yn(n-1) + 0.35*yn(n-2);
         if yn(n)>=0
             dn(n) = yn(n)/(3*sqrt(0.1 + 0.9*(yn(n)^2)));
         else
             dn(n) = -(yn(n)^2)*(1 - exp(0.7*yn(n)))/3;
         end        
        zn = sw*randn;
        d(n) = dn(n) + zn;
               
        % KLMS
        K = gram('gauss',un(:,n)',Dic',xi)';
        e(n) = d(n) - alpha'*K; 
        if max(K) < thresh
            alpha = [alpha; 0] + eta*e(n)*[K;1];
            Dic = [Dic,un(:,n)];
        else
            alpha = alpha + eta*e(n)*K;
        
        end
        
        % LMS
        en = d(n) - w'*un(:,n);             % estimation error
        exen = en - zn;                     % excess error              
        w = w + mu * un(:,n) * en;          % filter update
        e2(n) = en^2;        
    end
    
    % Simulation result saving (KLMS)
    waitbar(ind/rlz,sb)
    Ee2 = Ee2 + e.^2;
    
    % Simulation result saving (LMS)
    e2av = e2av + e2;
end
close(sb)

% Normalization KLMS
Ee2 = Ee2/rlz;
Ekd = Ekd/(rlz*(N-2));
Ed2 = Ed2/(rlz*(N-2));

% Normalization LMS
e2av = e2av/rlz;
exe2av = exe2av/rlz;



%%

%----------------------------------------------
%
% Part 3: Convergence analysis
%
%----------------------------------------------


% % Plot 1: LMS vs. KLMS
figure
hold on
plot(10*log10(filter(ones(1,1)/1,1,e2av(2:end))),'r')
plot(1:N,10*log10(filter(ones(1,1)/1,1,Ee2(1:N))),'b');
legend(['LMS'],['KLMS'])
xlabel('iteration','fontsize',14,'interpreter','Latex')
ylabel('MSE','fontsize',14,'interpreter','Latex')
title('LMS versus KLMS','fontsize',14,'interpreter','Latex')
grid


